#!/usr/bin/env python
# Copyright 2008-2009 David D Lowe
# released under the GPLv3

from pigment import PigmentType, Pigment
from pigment_more import FileBasedPigment, FileBasedPigmentType
import pigment
import sys, os
import gconf
import gtk
if __name__ == "__main__" or 'pychecker' in sys.argv[0]:
    sys.path.append("..")
import ep_exceptions
import hashlib
from gettext import gettext as _


class WallpaperType(FileBasedPigmentType):
    humanName = {"en":"Wallpaper"}
    humanPluralName = {"en":"Wallpapers"}
    codeName = "wallpaper"
    abbreviation = "wp"
    directoryName = "wallpapers"
    canGeneratePreview = True
    
    attachmentTypeHelp = _("The theme file should be an image file with the extension png , jpg , gif or svg")
        

    def get_Pigment(self):
        return Wallpaper

    publicBaseDir = "/usr/share/backgrounds/epidermis"
    personalBaseDir = "~/.local/share/backgrounds"


class Wallpaper(FileBasedPigment):
    
    
    def __init__(self):
        FileBasedPigment.__init__(self)
        self.type = WallpaperType()
        self.pictureOptions = ""
        self.colourShadingType = ""
        self.primaryColour = ""
        self.secondaryColour = ""
    
    def add_attachment(self, attachment, tempPath=None):
        try:
            FileBasedPigment.add_attachment(self, attachment, tempPath)
        except ep_exceptions.AttachmentNotFoundException, ee:
            # handle pigments saved in Epidermis 0.2.2 and before
            if os.path.exists(os.path.join(self.type.publicBaseDir, self.codeName, attachment)):
                raise ep_exceptions.PigmentMoreOldFormat(self, attachment)
            else:
                raise

    def upgrade_from_old_format(self, attachment, shell):
        shell.do(["mv", os.path.join(self.type.publicBaseDir, self.codeName, attachment),
            os.path.join(self.type.publicBaseDir, attachment)])
        shell.do(["rm", "-r", os.path.join(self.type.publicBaseDir, self.codeName)])
                
    def activate(self, progressCallback=None, shell=None):
        client = gconf.client_get_default()
        if progressCallback is None:
            progressCallback = lambda(a1): False
        if progressCallback(0.0): return
        if os.path.exists(os.path.join(self.type.publicBaseDir, self.attFilename)) and self.attFilename != "":
            client.set_string("/desktop/gnome/background/picture_filename", \
                os.path.join(self.type.publicBaseDir, self.attFilename))
        elif self.attFilename != "":
            client.set_string("/desktop/gnome/background/picture_filename", "")
        if progressCallback(0.5): return
        client.set_string("/desktop/gnome/background/picture_options", self.pictureOptions)
        client.set_string("/desktop/gnome/background/color_shading_type", self.colourShadingType)
        client.set_string("/desktop/gnome/background/primary_color", self.primaryColour)
        if self.secondaryColour != "":
            client.set_string("/desktop/gnome/background/secondary_color", self.secondaryColour)
        progressCallback(1.0)
    
    def read(self, filePath):
        FileBasedPigment.read(self, filePath)
        
        from xml.dom import minidom
        doc = minidom.parse(filePath)
        root = doc.getElementsByTagName("pigment")[0]
        
        
        for xmlName, var in (("pictureOptions", "pictureOptions"), \
        ("colorShadingType", "colourShadingType"), ("primaryColor", "primaryColour")):
            setattr(self, var, pigment.get_first_text(root.getElementsByTagName(xmlName)[0]))
        
        if len(root.getElementsByTagName("secondaryColor")) == 1:
            self.secondaryColour = pigment.get_first_text(root.getElementsByTagName("secondaryColor")[0])
        else:
            self.secondaryColour = ""
        
    def information_xml(self):
        """Returns the XML document describing the pigment"""
        doc = FileBasedPigment.information_xml(self)
        root = doc.getElementsByTagName("pigment")[0]
        
        bitsToAdd = [("pictureOptions", self.pictureOptions), \
        ("colorShadingType", self.colourShadingType), \
        ("primaryColor", self.primaryColour)]
        if self.secondaryColour != "":
            bitsToAdd.append(("secondaryColor", self.secondaryColour))
        
        for xmlName, varr in bitsToAdd:
            xmlBit = doc.createElement(xmlName)
            tn = doc.createTextNode(varr)
            xmlBit.appendChild(tn)
            root.appendChild(xmlBit)
        
        return doc
    
    
    def is_activated(self):
        """Returns whether this pigment is applied. To check if the effects
        have been activated regardless of Epidermis' pigments, use is_meat_activated"""
        if self.is_meat_activated():
            client = gconf.engine_get_default()
            if self.pictureOptions != client.get_string("/desktop/gnome/background/picture_options") or \
            self.colourShadingType != client.get_string("/desktop/gnome/background/color_shading_type") or \
            self.primaryColour != client.get_string("/desktop/gnome/background/primary_color"):
                return False
            if self.secondaryColour != "" and self.secondaryColour != client.get_string("/desktop/gnome/background/secondary_color"):
                return False
            return True
        else:
            return False

    def is_attachment_correct(self, attachmentIndex=None):
        if FileBasedPigment.is_attachment_correct(self, attachmentIndex):
            if attachmentIndex is None:
                attachmentIndex = 0
            if self.attachments[attachmentIndex] == "":
                return True
            extension = self.attachments[attachmentIndex].split(".")[-1]
            if not extension.lower() in ("png", "jpg", "svg"):
                return False
            return True
        else: return False

    
    def is_meat_activated(self):
        """Returns whether the meat of this pigment or similar to it is activated"""
        (md1, md2) = (hashlib.md5(), hashlib.md5())
        client = gconf.client_get_default()
        try:
            f1 = file(self.attPath, 'rb')
            f2 = file(client.get_string("/desktop/gnome/background/picture_filename"), 'rb')
        except:
            return False
        for mm, ff in ((md1, f1), (md2, f2)):
            ss = ff.read(8096)
            if not ss:
                break
            mm.update(ss)
        return m1 == m2
    
    def generate_preview(self):
        if not os.path.exists(self.attPath):
            return None
        pb = gtk.gdk.pixbuf_new_from_file_at_size(self.attPath,  100,  75)
        if pb is None:
            return None
        else:
            self.preview = pb
            return pb
    
    def does_activation_require_root(self):
        """Returns whether the activation requires root rights.
        If it does, the pigment object will need a pexpect bash shell as root
        to be passed to some of its methods"""
        return False
    
    def get_display_data(self, index=0):
        """Returns the object of index to be added to a gtk.ListStore, this
        could be a string, a gtk.gdk.Pixbuf or a custom widget, it should
        be rendered using type.create_cell_renderer"""
        return

def main():
    wp = Wallpaper()
    print wp.information_xml().toxml()
    print wp.previewFilename

if __name__ == "__main__":
    main()
